@@ -10,6 +10,7 @@
 #include <qpdf/QUtil.hh>
 #include <qpdf/PCRE.hh>
 #include <qpdf/Pipeline.hh>
+#include <qpdf/Pl_Discard.hh>
 
 #include <qpdf/QPDFExc.hh>
 #include <qpdf/QPDF_Null.hh>
@@ -435,7 +436,7 @@ QPDF::reconstruct_xref(QPDFExc& e)
 		int obj = atoi(m.getMatch(1).c_str());
 		int gen = atoi(m.getMatch(2).c_str());
 		int offset = this->file.getLastOffset();
-		insertXrefEntry(obj, 1, offset, gen);
+		insertXrefEntry(obj, 1, offset, gen, true);
 	    }
 	    else if ((! this->trailer.isInitialized()) &&
 		     trailer_re.match(line.c_str()))
@@ -465,7 +466,7 @@ QPDF::reconstruct_xref(QPDFExc& e)
 	// streams.
 
 	throw QPDFExc(this->file.getName() + ": unable to find trailer "
-		      "dictionary while recovering damanged file");
+		      "dictionary while recovering damaged file");
     }
 
     // We could iterate through the objects looking for streams and
@@ -865,11 +866,15 @@ QPDF::processXRefStream(off_t xref_offset, QPDFObjectHandle& xref_obj)
 }
 
 void
-QPDF::insertXrefEntry(int obj, int f0, int f1, int f2)
+QPDF::insertXrefEntry(int obj, int f0, int f1, int f2, bool overwrite)
 {
     // Populate the xref table in such a way that the first reference
     // to an object that we see, which is the one in the latest xref
-    // table in which it appears, is the one that gets stored.
+    // table in which it appears, is the one that gets stored.  This
+    // works because we are reading more recent appends before older
+    // ones.  Exception: if overwrite is true, then replace any
+    // existing object.  This is used in xref recovery mode, which
+    // reads the file from beginning to end.
 
     // If there is already an entry for this object and generation in
     // the table, it means that a later xref table has registered this
@@ -879,8 +884,16 @@ QPDF::insertXrefEntry(int obj, int f0, int f1, int f2)
 	ObjGen og(obj, gen);
 	if (this->xref_table.count(og))
 	{
-	    QTC::TC("qpdf", "QPDF xref reused object");
-	    return;
+	    if (overwrite)
+	    {
+		QTC::TC("qpdf", "QPDF xref overwrite object");
+		this->xref_table.erase(og);
+	    }
+	    else
+	    {
+		QTC::TC("qpdf", "QPDF xref reused object");
+		return;
+	    }
 	}
 	if (this->deleted_objects.count(obj))
 	{
@@ -1798,23 +1811,51 @@ QPDF::pipeStreamData(int objid, int generation,
 	}
     }
 
-    this->file.seek(offset, SEEK_SET);
-    char buf[10240];
-    while (length > 0)
+    try
     {
-	size_t to_read = (sizeof(buf) < length ? sizeof(buf) : length);
-	size_t len = this->file.read(buf, to_read);
-	if (len == 0)
+	this->file.seek(offset, SEEK_SET);
+	char buf[10240];
+	while (length > 0)
 	{
-	    throw QPDFExc(this->file.getName(), this->file.getLastOffset(),
-			  "unexpected EOF reading stream data");
+	    size_t to_read = (sizeof(buf) < length ? sizeof(buf) : length);
+	    size_t len = this->file.read(buf, to_read);
+	    if (len == 0)
+	    {
+		throw QPDFExc(this->file.getName(), this->file.getLastOffset(),
+			      "unexpected EOF reading stream data");
+	    }
+	    length -= len;
+	    pipeline->write((unsigned char*)buf, len);
 	}
-	length -= len;
-	pipeline->write((unsigned char*)buf, len);
+    }
+    catch (QEXC::General& e)
+    {
+	QTC::TC("qpdf", "QPDF decoding error warning");
+	warn(QPDFExc(this->file.getName(), this->file.getLastOffset(),
+		     "error decoding stream data for object " +
+		     QUtil::int_to_string(objid) + " " +
+		     QUtil::int_to_string(generation) + ": " + e.unparse()));
     }
     pipeline->finish();
 }
 
+void
+QPDF::decodeStreams()
+{
+    for (std::map<ObjGen, QPDFXRefEntry>::iterator iter =
+	     this->xref_table.begin();
+	 iter != this->xref_table.end(); ++iter)
+    {
+	ObjGen const& og = (*iter).first;
+	QPDFObjectHandle obj = getObjectByID(og.obj, og.gen);
+	if (obj.isStream())
+	{
+	    Pl_Discard pl;
+	    obj.pipeStreamData(&pl, true, false, false);
+	}
+    }
+}
+
 std::vector<QPDFObjectHandle> const&
 QPDF::getAllPages()
 {
