@@ -32,6 +32,7 @@ QPDFWriter::QPDFWriter(QPDF& pdf, char const* filename) :
     stream_data_mode(s_compress),
     qdf_mode(false),
     static_id(false),
+    suppress_original_object_ids(false),
     direct_stream_lengths(true),
     encrypted(false),
     preserve_encryption(true),
@@ -104,6 +105,12 @@ QPDFWriter::setStaticID(bool val)
     this->static_id = val;
 }
 
+void
+QPDFWriter::setSuppressOriginalObjectIDs(bool val)
+{
+    this->suppress_original_object_ids = val;
+}
+
 void
 QPDFWriter::setPreserveEncryption(bool val)
 {
@@ -160,7 +167,7 @@ QPDFWriter::setR3EncryptionParameters(
     //   None
     //   Only Document Assembly
     //   Only Form Field Fill-in or Signing
-    //   Comment AUthoring, Form Field Fill-in or Signing
+    //   Comment Authoring, Form Field Fill-in or Signing
     //   General Editing, Comment and Form Field Authoring
 
     // Allowed printing menu:
@@ -197,7 +204,7 @@ QPDFWriter::setR3EncryptionParameters(
     switch (modify)
     {
       case r3m_none:
-	clear.insert(11);	// document essembly
+	clear.insert(11);	// document assembly
 
       case r3m_assembly:
 	clear.insert(9);	// filling in form fields
@@ -226,7 +233,7 @@ QPDFWriter::setEncryptionParameters(
     // PDF specification refers to bits with the low bit numbered 1.
     // We have to convert this into a bit field.
 
-    // Specification always requirse bits 1 and 2 to be cleared.
+    // Specification always requires bits 1 and 2 to be cleared.
     bits_to_clear.insert(1);
     bits_to_clear.insert(2);
 
@@ -754,7 +761,7 @@ QPDFWriter::unparseObject(QPDFObjectHandle object, int level,
 	{
 	    // Don't filter if the stream is already compressed with
 	    // FlateDecode.  We don't want to make it worse by getting
-	    // rid of a predictor or otherwising messing with it.  We
+	    // rid of a predictor or otherwise messing with it.  We
 	    // should also avoid messing with anything that's
 	    // compressed with a lossy compression scheme, but we
 	    // don't support any of those right now.
@@ -932,7 +939,13 @@ QPDFWriter::writeObjectStream(QPDFObjectHandle object)
 	    {
 		writeString("%% Object stream: object " +
 			    QUtil::int_to_string(new_obj) + ", index " +
-			    QUtil::int_to_string(count) + "\n");
+			    QUtil::int_to_string(count));
+		if (! this->suppress_original_object_ids)
+		{
+		    writeString("; original object ID: " +
+				QUtil::int_to_string(obj));
+		}
+		writeString("\n");
 	    }
 	    if (pass == 1)
 	    {
@@ -1026,6 +1039,12 @@ QPDFWriter::writeObject(QPDFObjectHandle object, int object_stream_index)
     }
     if (object_stream_index == -1)
     {
+	if (this->qdf_mode && (! this->suppress_original_object_ids))
+	{
+	    writeString("%% Original object ID: " +
+			QUtil::int_to_string(object.getObjectID()) + " " +
+			QUtil::int_to_string(object.getGeneration()) + "\n");
+	}
 	openObject(new_id);
 	setDataKey(new_id);
 	unparseObject(object, 0, 0);
