@@ -12,6 +12,9 @@
 
 #include <qpdf/QPDFWriter.hh>
 
+static int const EXIT_ERROR = 2;
+static int const EXIT_WARNING = 3;
+
 static char const* whoami = 0;
 
 // Note: let's not be too noisy about documenting the fact that this
@@ -107,7 +110,7 @@ Values for stream data options:\n\
   preserve              leave all stream data as is\n\
   uncompress            uncompress stream data when possible\n\
 \n\
-Values for object strea mode:\n\
+Values for object stream mode:\n\
 \n\
   preserve                  preserve original object streams (default)\n\
   disable                   don't write any object streams\n\
@@ -124,6 +127,7 @@ These options can be useful for digging into PDF files or for use in\n\
 automated test suites for software that uses the qpdf library.\n\
 \n\
 --static-id               generate static /ID: FOR TESTING ONLY!\n\
+--no-original-object-ids  suppress original object ID comments in qdf mode\n\
 --show-encryption         quickly show encryption parameters\n\
 --check-linearization     check file integrity and linearization status\n\
 --show-linearization      check and show all linearization data\n\
@@ -142,8 +146,11 @@ stream data to be written to standard output.\n\
 If --filtered-stream-data is given and --normalize-content=y is also\n\
 given, qpdf will attempt to normalize the stream data as if it is a\n\
 page content stream.  This attempt will be made even if it is not a\n\
-page content stream, in which case it will produce unusuable results.\n\
+page content stream, in which case it will produce unusable results.\n\
 \n\
+Ordinarily, qpdf exits with a status of 0 on success or a status of 2\n\
+if any errors occurred.  In --check mode, if there were warnings but not\n\
+errors, qpdf exits with a status of 3.\n\
 \n";
 
 void usage(std::string const& msg)
@@ -155,7 +162,7 @@ void usage(std::string const& msg)
 	<< "Usage: " << whoami << " [options] infile outfile" << std::endl
 	<< "For detailed help, run " << whoami << " --help" << std::endl
 	<< std::endl;
-    exit(2);
+    exit(EXIT_ERROR);
 }
 
 static void show_encryption(QPDF& pdf)
@@ -436,7 +443,9 @@ int main(int argc, char* argv[])
 	whoami += 3;
     }
 
-    if ((argc == 2) && (strcmp(argv[1], "--version") == 0))
+    if ((argc == 2) &&
+	((strcmp(argv[1], "--version") == 0) ||
+	 (strcmp(argv[1], "-version") == 0)))
     {
 	// make_dist looks for the line of code here that actually
 	// prints the version number, so read make_dist if you change
@@ -447,8 +456,8 @@ int main(int argc, char* argv[])
 	//               1         2         3         4         5         6         7         8
 	//      12345678901234567890123456789012345678901234567890123456789012345678901234567890
 	std::cout
-	    << whoami << " version 2.0.2" << std::endl
-	    << "Copyright (c) 2005-2008 Jay Berkenbilt"
+	    << whoami << " version 2.0.6" << std::endl
+	    << "Copyright (c) 2005-2009 Jay Berkenbilt"
 	    << std::endl
 	    << "This software may be distributed under the terms of version 2 of the"
 	    << std::endl
@@ -459,7 +468,9 @@ int main(int argc, char* argv[])
 	exit(0);
     }
 
-    if ((argc == 2) && (strcmp(argv[1], "--help") == 0))
+    if ((argc == 2) &&
+	((strcmp(argv[1], "--help") == 0) ||
+	 (strcmp(argv[1], "-help") == 0)))
     {
 	std::cout << help;
 	exit(0);
@@ -493,6 +504,7 @@ int main(int argc, char* argv[])
     bool qdf_mode = false;
 
     bool static_id = false;
+    bool suppress_original_object_id = false;
     bool show_encryption = false;
     bool check_linearization = false;
     bool show_linearization = false;
@@ -627,6 +639,10 @@ int main(int argc, char* argv[])
 	    {
 		static_id = true;
 	    }
+	    else if (strcmp(arg, "no-original-object-ids") == 0)
+	    {
+		suppress_original_object_id = true;
+	    }
 	    else if (strcmp(arg, "show-encryption") == 0)
 	    {
 		show_encryption = true;
@@ -745,7 +761,7 @@ int main(int argc, char* argv[])
 		}
 		else
 		{
-		    exit(2);
+		    exit(EXIT_ERROR);
 		}
 	    }
 	    if (show_linearization)
@@ -770,7 +786,7 @@ int main(int argc, char* argv[])
 			    QTC::TC("qpdf", "unable to filter");
 			    std::cerr << "Unable to filter stream data."
 				      << std::endl;
-			    exit(2);
+			    exit(EXIT_ERROR);
 			}
 			else
 			{
@@ -862,6 +878,8 @@ int main(int argc, char* argv[])
 			// traversal of file, so any structural errors
 			// would be exposed.
 			pdf.flattenScalarReferences();
+			// Also explicitly decode all streams.
+			pdf.decodeStreams();
 			okay = true;
 		    }
 		}
@@ -871,7 +889,14 @@ int main(int argc, char* argv[])
 		}
 		if (okay)
 		{
-		    std::cout << "No errors found" << std::endl;
+		    if (! pdf.getWarnings().empty())
+		    {
+			exit(EXIT_WARNING);
+		    }
+		    else
+		    {
+			std::cout << "No errors found" << std::endl;
+		    }
 		}
 	    }
 	}
@@ -902,6 +927,10 @@ int main(int argc, char* argv[])
 	    {
 		w.setStaticID(true);
 	    }
+	    if (suppress_original_object_id)
+	    {
+		w.setSuppressOriginalObjectIDs(true);
+	    }
 	    if (encrypt)
 	    {
 		if (keylen == 40)
@@ -931,11 +960,17 @@ int main(int argc, char* argv[])
 	    }
 	    w.write();
 	}
+	if (! pdf.getWarnings().empty())
+	{
+	    std::cerr << whoami << ": operation succeeded with warnings;"
+		      << " resulting file may have some problems" << std::endl;
+	    exit(EXIT_WARNING);
+	}
     }
     catch (std::exception& e)
     {
 	std::cerr << e.what() << std::endl;
-	exit(2);
+	exit(EXIT_ERROR);
     }
 
     return 0;
